/*

  gLogger
    
	Martin Matysiak, 2008
	    mail@k621.de

  Version 1.01

*/

#define F_CPU 7372800UL

#define IO_PORT PORTA
#define IO_CONF DDRA
#define IO_INPUT PINA

#define EXT_PORT PORTC
#define EXT_CONF DDRC

#define LED_BATT PA4
#define LED_WARN PA5
#define LED_STAT PA6

#define LEDCODE_OFF IO_PORT &= ~((1 << LED_WARN) | (1 << LED_STAT))
#define LEDCODE_OK IO_PORT = (IO_PORT & ~(1 << LED_WARN)) | (1 << LED_STAT)
#define LEDCODE_WARNING IO_PORT = (IO_PORT & ~(1 << LED_STAT)) | (1 << LED_WARN)
#define LEDCODE_PROCESSING IO_PORT |= (1 << LED_WARN) | (1 << LED_STAT)
#define LEDCODE_BLINK IO_PORT ^= (1 << LED_STAT)

#define BTN_RECORD PA7

#define BAT_CHAN PA0

#include <avr/io.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <gps.h>
#include <mmc.h>
#include <fat16.h>
 
struct {
	unsigned record:1;
	unsigned keydown:2;
} status;

volatile uint16_t ledCount;

uint16_t ReadChannel(uint8_t mux)
{
  uint8_t i;
  uint16_t result;
 
  ADMUX = mux;                      // Kanal waehlen
  ADMUX |= (1<<REFS1) | (1<<REFS0); // interne Referenzspannung nutzen
 
  ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1);    // Frequenzvorteiler 
                               // setzen auf 8 (1) und ADC aktivieren (1)
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
  ADCSRA |= (1<<ADSC);              // eine ADC-Wandlung 
  while ( ADCSRA & (1<<ADSC) ) {
     ;     // auf Abschluss der Konvertierung warten 
  }
  result = ADCW;  // ADCW muss einmal gelesen werden,
                  // sonst wird Ergebnis der nchsten Wandlung
                  // nicht bernommen.
 
  /* Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen */
  result = 0; 
  for( i=0; i<4; i++ )
  {
    ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
    while ( ADCSRA & (1<<ADSC) ) {
      ;   // auf Abschluss der Konvertierung warten
    }
    result += ADCW;		    // Wandlungsergebnisse aufaddieren
  }
  ADCSRA &= ~(1<<ADEN);             // ADC deaktivieren (2)
 
  result /= 4;                     // Summe durch vier teilen = arithm. Mittelwert
 
  return result;
}

ISR(TIMER0_OVF_vect) {
	ledCount++;

	if (ledCount > 64) {
		LEDCODE_BLINK;
		ledCount = 0;
	}
}

int main (void) {

	//Interrupts global aktivieren
	sei();
 
	//Ports definieren
	IO_CONF |= (1 << LED_BATT) | (1 << LED_WARN) | (1 << LED_STAT);
	LEDCODE_PROCESSING;

	//SD Karte + FAT initalisieren
	if ((mmc_init() != 0) || (InitFat16() != 0)) {
		LEDCODE_WARNING;
		while(1);
	}

	LEDCODE_OK;

	//GPS Modul initalisieren, auf Fix warten
	gps_init();

	//Variablen initalisieren
	File logFile;
	LEDCODE_OFF;
	status.record = 0;
	char incoming[128];
	char counter = 0;
	uint16_t bat = 1024;

	//Hauptschleife
	while(1) {
		if (~IO_INPUT & (1 << BTN_RECORD)) {
			LEDCODE_WARNING;
			if (!status.keydown) {
				status.keydown = 1;
				_delay_ms(2000);
			} else {
				LEDCODE_PROCESSING;
				status.record = !status.record;
				if(status.record) {
					//Timer fr LED aktivieren
					ledCount = 0;
					TCCR0 |= (1 << CS02) | (1 << CS00);
					TIMSK |= (1 << TOIE0);

					fopen_((unsigned char*)"log.nmea",'a',&logFile);
					fputs_(&logFile,"***START***");
				} else {
					fputs_(&logFile,"***STOP***");
					fclose_(&logFile);

					TIMSK &= ~(1 << TOIE0);
					TCCR0 &= 0x00;
				}
				_delay_ms(500);
				LEDCODE_OFF;
			}
		} else {
			status.keydown = 0;
			LEDCODE_OFF;
		}

		if (status.record) {
		    gps_get_nmea(incoming, sizeof(incoming));
			fputs_(&logFile, incoming);
		} else {
			_delay_ms(950);
			LEDCODE_WARNING;
			_delay_ms(50);
			LEDCODE_OFF;
		}

		counter++;

		if (counter >= 100) {

			counter = 0;
		
			bat = ReadChannel(BAT_CHAN);

			if (bat < 550) {
				IO_PORT ^= (1 << LED_BATT);
			}

			if (bat < 540) {
				//Aufnahme stoppen (wenn aktiv)
				if (status.record) {
					fputs_(&logFile,"***BATT LOW***");
					fclose_(&logFile);
					TIMSK &= ~(1 << TOIE0);
					TCCR0 &= 0x00;
					status.record = 0;
					_delay_ms(500);
				}

				//cli();

				LEDCODE_OFF;
				IO_PORT |= (1 << LED_BATT);
				set_sleep_mode(SLEEP_MODE_PWR_DOWN);
				sleep_mode();
			}
		}
	}
 
	/* wird nie erreicht */
	return 0;
}
