#ifndef __fat16_h
#define __fat16_h

//________________________________________________________________________________________________________________________________________
// 
// Structure of a filepointer
//				
//________________________________________________________________________________________________________________________________________

typedef struct afile
{
	unsigned long 	start_cluster;				// Sectorpointer to the first sector of the first datacluster of the file. 
	unsigned long 	cluster_pointer;			// Pointer to the cluster which is edited at the moment.
	unsigned char 	sector_index;				// The sector which is edited at the moment (cluster_pointer + sector_index).
	unsigned int  	byte_index;					// The bytelocation within the current sector (cluster_pointer + sector_index + byte_index).
	unsigned char 	mode;						// mode of fileoperation (read,write)
	unsigned long  filesize;					// the size of the opend file in bytes.
	unsigned long  fileposition;				// pointer to a character within the file 0 < fileposition < filesize
	unsigned long  sector_in_buffer;			// the last sector read, wich is still in the sectorbuffer.
	unsigned long  directory_sector;			// the sectorposition where the directoryentry has been made.
	unsigned char  directory_index;			// the index to the directoryentry within the specified sector.
	unsigned char  attribute;					// the attribute of the file opened.
} File;

//________________________________________________________________________________________________________________________________________
// 
// Structure of an directoryentry
//				
//________________________________________________________________________________________________________________________________________

struct DirEntry
{
	unsigned char name[8];						// 8 bytes name.
	unsigned char extension[3];				// 3 bytes extension.
	unsigned char attribute;					// attribute of the directory entry (unused,archive,read-only,system,directory,volume)
	unsigned char reserved[10];
	unsigned int  time;						// time and
	unsigned int  date;						// date of last write acces to the file or directory.
	unsigned int  startcluster;				// first cluster of the file or directory.
	unsigned long size;						// size of the file or directory in bytes.
};

//________________________________________________________________________________________________________________________________________
// 
// Structure of an entry within the fileallocationtable.
//				
//________________________________________________________________________________________________________________________________________

struct FatEntry
{
	unsigned int  next_cluster;				// the next cluster of the file.
};
//________________________________________________________________________________________________________________________________________
// 
// API to the FAT16 filesystem
//				
//________________________________________________________________________________________________________________________________________

extern unsigned char 	InitFat16(void);		
extern unsigned char 	fopen_(unsigned char *fname,char mode, File *file);
extern int				fflush_(File *file);
extern void 			fclose_(File *file);
extern unsigned long 	fread_(void *buffer, unsigned long size, unsigned long count, File *file); 
extern unsigned long 	fwrite_(void *buffer, unsigned long size, unsigned long count, File *file);
extern int  			fseek_(File *file, long offset, int origin);
extern 	         int 	fgetchar_(File *file);
extern unsigned char 	fputchar_(File *file,char c);
extern unsigned char 	fputs_(File *file,char *string);
extern char *  		fgets_(char *s, int count, File *file);
extern int				rename(char *oldname, char *newname);
extern unsigned char  fexist_(unsigned char *fname, File *file);

//________________________________________________________________________________________________________________________________________
// 
// Functions needed internaly for the fat16 implementation 
//				
//________________________________________________________________________________________________________________________________________

extern unsigned char 	CreateDirectoryEntry(unsigned char *fname, unsigned int cluster, File *file,unsigned char attrib);
extern unsigned int 	FindNextFreeCluster(File *file);
extern unsigned char 	SeekDirectoryEntry(unsigned char *fname, File *file);
extern void 			SeperateFileName(unsigned char *fname, unsigned char *name);
extern unsigned char  ScanSubDirectories(unsigned char *fname, File *file);
extern unsigned int   GetNextCluster(File *file);
extern unsigned char  AppendCluster(File *file);
extern unsigned int 	GetFatClusterOffset(File *file);
extern unsigned int 	GetFatSectorIndex(File *file);


//________________________________________________________________________________________________________________________________________
// 
// Definitions
//				
//________________________________________________________________________________________________________________________________________

//#define		__USE_TIME_DATE_ATTRIBUTE
#define		MBR_SECTOR					0		// the masterboot record is located in sector 0.
#define		_UNUSED		1						// Bits used in the attribute of an directory entry.
#define		_ARCHIVE	2
#define		_READ_ONLY	4
#define		_SYSTEM		8
#define		_DIRECTORY	16
#define		_FILE		32

//________________________________________________________________________________________________________________________________________
// 
// Vaiables needed internaly for the fat16 implementation 
//				
//________________________________________________________________________________________________________________________________________

extern unsigned char 	SectorsPerCluster;
extern unsigned char 	FileBuffer[512];
extern unsigned char 	SectorsPerCluster;
extern File myfile;

//extern struct   time 	rtctime;
#endif




